﻿using System;
using System.Drawing;

//====================================================
// Описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
// Автор исходных кодов Клочков Павел
//====================================================

namespace linesk
{
    ///////////////////////////////////////////////////////////////////////////
    //    Модуль настроек приложения и глобальных переменных.
    ///////////////////////////////////////////////////////////////////////////


    // Глобальные данные приложения.
    public class Global
    {
        /// <summary>
        /// Количество цветов графэлеменов
        /// </summary>
        public const int AmountColorGraphItms = 3;

        /// <summary>
        /// количество графэлементов
        /// </summary>
        public const int NumGraphItems = 100;

        /// <summary>
        /// Количество запоминаемых рекордсменов
        /// </summary>
        public const int NumPlayers = 6;
    }


    /// <summary>
    /// Класс для хранения и записи настроек в двоичный файл.
    /// Обязательно устанавливаем атрибут [Serializable]
    /// </summary>
    [Serializable]
    class GameSetting
    {
        public GameSetting()
        {
            // Инициализация игроков рекордсменов.
            for (int i = 0; i < DRH.Length; i++)
            {
                DRH[i] = new DataRecordsman();
            }

            // Цвета графэлементов по умолчанию.
            GraphItemsColors = new Color[] { Color.DeepPink, Color.DeepSkyBlue, Color.Gold };
        }

        /// <summary>
        /// Хранение данных рекордсменов.
        /// </summary>
        public DataRecordsman[] DRH = new DataRecordsman[Global.NumPlayers];

        /// <summary>
        /// Хранение цвета графэлементов.
        /// </summary>
        public Color[] GraphItemsColors = null;

        /// <summary>
        /// Полноэкранный вид окна игры.
        /// </summary>
        public bool FullScreen = false;

        /// <summary>
        /// Размер и положение окна приложения.
        /// </summary>
        public Rectangle Bounds;

        /// <summary>
        /// Текущий вид графэлемента(шар, прямоугольник, ромб).
        /// </summary>
        public GraphItem.TypeGraphItem CurrentGraphItemType;
    }



    /// <summary>
    /// Данные игроков-рекордсменов для сериализации (сохранения в файл).
    /// </summary>
    [Serializable]
    public class DataRecordsman
    {
        // очки
        public int Score = 0;

        // имя игрока
        public string Name = null;

        // действующий игрок
        public bool CurrentPlayer = false;
    }

}
